/*
    MPEG Maaate: An Australian MPEG audio analysis toolkit
    Copyright (C) 2000 Commonwealth Scientific and Industrial Research Organisation
    (CSIRO), Australia.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <math.h>
#include "SOUNDfile.H"
#include "tools.H"

#define SECURE 1
//set it to 0 to remove test at the begining of functions

/************************* Other tools **********************/


/*------------------------ search_array --------------------*/

CSAPI_PLUGINS
int search_array ( double tab[], double what, int lenght) {

  int where;
  int from = 0;
  int to = lenght-1;

#ifdef SECURE
  if ( tab == NULL ) {
    cerr << "ToolsLibrary: array pointer NULL" << endl;
    return 0;
  }

  if (lenght <= 0) {
    cerr << "ToolsLibrary: wrong lenght" << endl;
  }

#endif

  //finding the 2 nearest values
  while (to != (from+1)) {
    where = from + (to-from)/2;
    if ( what >= tab[where]) {
      from = where;
    } else {
      to = where;
    }
  }

  if ( tab[from] == what ) {
    return from;
  } else {
    return to  ;
  }

}


/****************************** windows functions ***********************************/

/*---------------------- square_window --------------------*/
CSAPI_PLUGINS
double square_window ( unsigned int upto, int x) {

  if ((x >= 0) &&( x <= (int) upto)) {
    return 1.0;
  } else {
    return 0.0;
  }
}


/*---------------------- Hamming window ----------------------*/
CSAPI_PLUGINS
double hamming_window ( unsigned int upto, int x) {

  if ((x >= 0) &&( x <= (int) upto)) {
    return 0.54 - 0.46 * cos( 2 * PI * x/upto);
  } else {
    return 0.0;
  }
}


/*---------------------- Welch window ----------------------*/
CSAPI_PLUGINS
double welch_window ( unsigned int upto, int x) {

  if ((x >= 0) &&( x <= (int) upto)) {
    return 1 - pow( ((x - 0.5*upto) / (0.5 * upto)) , 2 );
  } else {
    return 0.0;
  }
}


/*---------------------- Bartlett window ----------------------*/
CSAPI_PLUGINS
double bartlett_window ( unsigned int upto, int x) {

  if ((x >= 0) &&( x <= (int) upto)) {
    return  1 - fabs( ((x - 0.5*upto) / (0.5 * upto)) );
  } else {
    return 0.0;
  }
}

