/*
    MPEG Maaate: An Australian MPEG audio analysis toolkit
    Copyright (C) 2000 Commonwealth Scientific and Industrial Research Organisation
    (CSIRO), Australia.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "maaateM_brief.H"


using namespace std;

CSAPI_PLUGINS_BRIEF SegmentData * 
variance(Plugins * plugins, SegmentData *sd, 
       double from=0.0, double to=FLT_MAX, 
       double dur = 1.0) {

  // calculate low Nrj
  Module * m = plugins->GetModule(string("variance"));
  if (m==NULL) {
    return NULL;
  }

  // get default values
  list<ModuleParam> * paramsIn = m->defaultValues();
  if (paramsIn->size() == 0) {
    return NULL;
  }

  // set parameters
  list<ModuleParam>::iterator iter;
  iter=paramsIn->begin();
  (*iter).set(sd); // set the Segment Data ptr.
  ++iter; (*iter).set(from);
  ++iter; (*iter).set(to);
  ++iter; (*iter).set(dur);

  // suggest and check parameters
  m->suggestValues(paramsIn);

  // execute module
  list<ModuleParam> * paramsOut = m->apply(paramsIn);
  if (paramsOut->size() == 0) {
    return NULL;
  }

  return paramsOut->front().get_sd();
}

CSAPI_PLUGINS_BRIEF SegmentData * 
sumScalefactors(Plugins * plugins, SOUNDfile *sfile, 
		double from=0.0, double to=FLT_MAX, 
		int fromSb=0, int toSb=575) {

  // calculate sum of scalefactors (loudness approx)
  Module * m = plugins->GetModule(string("sumscf"));
  if (m==NULL) {
    return NULL;
  }

  // get default values
  list<ModuleParam> * paramsIn = m->defaultValues();
  if (paramsIn->size() == 0) {
    return NULL;
  }

  // set parameters
  list<ModuleParam>::iterator iter;
  iter=paramsIn->begin();
  (*iter).set(sfile); // set the Soundfile ptr.
  ++iter; (*iter).set(from);
  ++iter; (*iter).set(to);
  ++iter; (*iter).set(fromSb);
  ++iter; (*iter).set(toSb);

  // suggest and check parameters
  m->suggestValues(paramsIn);

  // execute module
  list<ModuleParam> * paramsOut = m->apply(paramsIn);
  if (paramsOut->size() == 0) {
    return NULL;
  }

  return paramsOut->front().get_sd();
}

CSAPI_PLUGINS_BRIEF SegmentData * 
lowNrj(Plugins * plugins, SegmentData *sd, 
       double from=0.0, double to=FLT_MAX, 
       double dur = 1.0) {

  // calculate low Nrj
  Module * m = plugins->GetModule(string("lowNrj"));
  if (m==NULL) {
    return NULL;
  }

  // get default values
  list<ModuleParam> * paramsIn = m->defaultValues();
  if (paramsIn->size() == 0) {
    return NULL;
  }

  // set parameters
  list<ModuleParam>::iterator iter;
  iter=paramsIn->begin();
  (*iter).set(sd); // set the Segment Data ptr.
  ++iter; (*iter).set(from);
  ++iter; (*iter).set(to);
  ++iter; (*iter).set(dur);

  // suggest and check parameters
  m->suggestValues(paramsIn);

  // execute module
  list<ModuleParam> * paramsOut = m->apply(paramsIn);
  if (paramsOut->size() == 0) {
    return NULL;
  }

  return paramsOut->front().get_sd();
}
    
CSAPI_PLUGINS_BRIEF SegmentData * 
spectralCentroid(Plugins * plugins, SOUNDfile *sfile, 
		 double from=0.0, double to=FLT_MAX, 
		 int fromSb=0, int toSb=575) {
  // calculate spectral centroid (brightness approx)
  Module * m = plugins->GetModule(string("centroid"));
  if (m==NULL) {
    return NULL;
  }

  // get default values
  list<ModuleParam> * paramsIn = m->defaultValues();
  if (paramsIn->size() == 0) {
    return NULL;
  }

  // set parameters
  list<ModuleParam>::iterator iter;
  iter=paramsIn->begin();
  (*iter).set(sfile); // set the Soundfile ptr.
  ++iter; (*iter).set(from);
  ++iter; (*iter).set(to);
  ++iter; (*iter).set(fromSb);
  ++iter; (*iter).set(toSb);

  // suggest and check parameters
  m->suggestValues(paramsIn);

  // execute module
  list<ModuleParam> * paramsOut = m->apply(paramsIn);
  if (paramsOut->size() == 0) {
    return NULL;
  }

  return paramsOut->front().get_sd();
}


CSAPI_PLUGINS_BRIEF double
backgroundNoiseLevel(Plugins * plugins, SegmentData *sd, 
		     double from=0.0, double to=FLT_MAX, 
		     double minDur=0.1, double maxInterrupt=0.0, 
		     double onsetTime=0.01, double releaseTime=0.01) {

  // calculate background noise level
  Module * m = plugins->GetModule(string("bgnoiselevel"));
  if (m==NULL) {
    return -1.0;
  }

  // get default values
  list<ModuleParam> * paramsIn = m->defaultValues();
  if (paramsIn->size() == 0) {
    return -1.0;
  }

  // set parameters
  list<ModuleParam>::iterator iter;
  iter=paramsIn->begin();
  (*iter).set(sd); // set the sumscf loudness approximation function
  ++iter; (*iter).set(from);
  ++iter; (*iter).set(to);
  ++iter; (*iter).set(minDur);
  ++iter; (*iter).set(maxInterrupt);
  ++iter; (*iter).set(onsetTime);
  ++iter; (*iter).set(releaseTime);

  // suggest and check parameters
  m->suggestValues(paramsIn);

  // execute module
  list<ModuleParam> * paramsOut = m->apply(paramsIn);
  if (paramsOut->size() == 0) {
    return -1.0;
  }

  return paramsOut->front().get_r();
}


CSAPI_PLUGINS_BRIEF SegmentTable *
silenceSegmentation(Plugins * plugins, SegmentData *sd, 
	double from=0.0, double to=FLT_MAX, 
	double thresh=0.01, double minDur=0.1, double maxInterrupt=0.0, 
	double onsetTime=0.01, double releaseTime=0.01) {

  // calculate silence segments
  Module * m = plugins->GetModule(string("silence"));
  if (m==NULL) {
    return NULL;
  }

  // get default values
  list<ModuleParam> * paramsIn = m->defaultValues();
  if (paramsIn->size() == 0) {
    return NULL;
  }

  // set parameters
  list<ModuleParam>::iterator iter;
  iter=paramsIn->begin();
  (*iter).set(sd); // set the sumscf loudness approximation function
  ++iter; (*iter).set(from);
  ++iter; (*iter).set(to);
  ++iter; (*iter).set(thresh);
  ++iter; (*iter).set(minDur);
  ++iter; (*iter).set(maxInterrupt);
  ++iter; (*iter).set(onsetTime);
  ++iter; (*iter).set(releaseTime);

  // suggest and check parameters
  m->suggestValues(paramsIn);
  if (paramsIn->size() == 0) {
    return NULL;
  }

  // execute module
  list<ModuleParam> * paramsOut = m->apply(paramsIn);
  if (paramsOut->size() == 0) {
    return NULL;
  }

  return paramsOut->front().get_st();
}


CSAPI_PLUGINS_BRIEF SegmentTable *
noiseSegmentation(Plugins * plugins, SegmentData *sd, 
       double from=0.0, double to=FLT_MAX, 
       double thresh=0.01, double minDur=0.1, double maxInterrupt=0.0, 
       double onsetTime=0.01, double releaseTime=0.01) {

  // calculate noise segmentation
  Module * m = plugins->GetModule(string("noise"));
  if (m==NULL) {
    return NULL;
  }

  // get default values
  list<ModuleParam> * paramsIn = m->defaultValues();
  if (paramsIn->size()==0) {
    return NULL;
  }

  // set parameters
  list<ModuleParam>::iterator iter;
  iter=paramsIn->begin();
  (*iter).set(sd); // set the sumscf loudness approximation function
  ++iter; (*iter).set(from);
  ++iter; (*iter).set(to);
  ++iter; (*iter).set(thresh);
  ++iter; (*iter).set(minDur);
  ++iter; (*iter).set(maxInterrupt);
  ++iter; (*iter).set(onsetTime);	
  ++iter; (*iter).set(releaseTime);

  // suggest and check parameters
  m->suggestValues(paramsIn);
  if (paramsIn->size() == 0) {
    return NULL;
  }

  // execute module
  list<ModuleParam> * paramsOut = m->apply(paramsIn);
  if (paramsOut->size() == 0) {
    return NULL;
  }

  return paramsOut->front().get_st();
}


CSAPI_PLUGINS_BRIEF SegmentTable *
segmentation(Plugins * plugins, SegmentData *sd, 
	     double from=0.0, double to=FLT_MAX, 
	     bool below=true, double thresh=0.01, 
	     double minDur=0.1, double maxInterrupt=0.0, 
	     double onsetTime=0.01, double releaseTime=0.01) {

  // calculate noise segmentation
  Module * m = plugins->GetModule(string("segmentation"));
  if (m==NULL) {
    return NULL;
  }
  
  // get default values
  list<ModuleParam> * paramsIn = m->defaultValues();
  if (paramsIn->size()==0) {
    return NULL;
  }
  
  // set parameters
  list<ModuleParam>::iterator iter;
  iter=paramsIn->begin();
  (*iter).set(sd);
  ++iter; (*iter).set(from);
  ++iter; (*iter).set(to);
  ++iter; (*iter).set(below);
  ++iter; (*iter).set(thresh);
  ++iter; (*iter).set(minDur);
  ++iter; (*iter).set(maxInterrupt);
  ++iter; (*iter).set(onsetTime);	
  ++iter; (*iter).set(releaseTime);

  // suggest and check parameters
  m->suggestValues(paramsIn);
  if (paramsIn->size() == 0) {
    return NULL;
  }

  // execute module
  list<ModuleParam> * paramsOut = m->apply(paramsIn);
  if (paramsOut->size() == 0) {
    return NULL;
  }

  return paramsOut->front().get_st();
}

    
CSAPI_PLUGINS_BRIEF ModuleParamList *
histo1D(Plugins * plugins, SegmentData *sd, 
      double from=0.0, double to=FLT_MAX, 
      int bins=10, double starthisto=DBL_MAX, double endhisto=DBL_MIN) {

  // calculate histogram
  Module * m = plugins->GetModule(string("histogram1D"));
  if (m==NULL) {
    return NULL;
  }

  // get default values
  list<ModuleParam> * paramsIn = m->defaultValues();
  if (paramsIn->size()==0) {
    return NULL;
  }
      
  // set parameters
  list<ModuleParam>::iterator iter;
  iter=paramsIn->begin();
  (*iter).set(sd);
  ++iter; (*iter).set(from);
  ++iter; (*iter).set(to);
  ++iter; (*iter).set(bins);
  ++iter; (*iter).set(starthisto);
  ++iter; (*iter).set(endhisto);
  
  // suggest and check parameters
  m->suggestValues(paramsIn);
  if (paramsIn->size() == 0) {
    return NULL;
  }
      
  // execute module
  return m->apply(paramsIn);
}


/*-------------- SBnrj -----------------------------*/
CSAPI_PLUGINS_BRIEF SegmentData * 
SBnrj(Plugins * plugins, SOUNDfile *sfile, 
		double from=0.0, double to=FLT_MAX, 
		int fromSb=0, int toSb=575) {

  // calculate normalised subband energies
  Module * m = plugins->GetModule(string("SBnrj"));
  if (m==NULL) {
    return NULL;
  }

  // get default values
  list<ModuleParam> * paramsIn = m->defaultValues();
  if (paramsIn->size() == 0) {
    return NULL;
  }

  // set parameters
  list<ModuleParam>::iterator iter;
  iter=paramsIn->begin();
  (*iter).set(sfile); // set the Soundfile ptr.
  ++iter; (*iter).set(from);
  ++iter; (*iter).set(to);
  ++iter; (*iter).set(fromSb);
  ++iter; (*iter).set(toSb);

  // suggest and check parameters
  m->suggestValues(paramsIn);

  // execute module
  list<ModuleParam> * paramsOut = m->apply(paramsIn);
  if (paramsOut->size() == 0) {
    return NULL;
  }

  return paramsOut->front().get_sd();
}

/*-------------- SBmean -----------------------------*/
CSAPI_PLUGINS_BRIEF SegmentData * 
SBmean(Plugins * plugins, SOUNDfile *sfile, 
		double from=0.0, double to=FLT_MAX, 
		int fromSb=0, int toSb=575) {

  // calculate normalised subband energies
  Module * m = plugins->GetModule(string("SBmean"));
  if (m==NULL) {
    return NULL;
  }

  // get default values
  list<ModuleParam> * paramsIn = m->defaultValues();
  if (paramsIn->size() == 0) {
    return NULL;
  }

  // set parameters
  list<ModuleParam>::iterator iter;
  iter=paramsIn->begin();
  (*iter).set(sfile); // set the Soundfile ptr.
  ++iter; (*iter).set(from);
  ++iter; (*iter).set(to);
  ++iter; (*iter).set(fromSb);
  ++iter; (*iter).set(toSb);

  // suggest and check parameters
  m->suggestValues(paramsIn);

  // execute module
  list<ModuleParam> * paramsOut = m->apply(paramsIn);
  if (paramsOut->size() == 0) {
    return NULL;
  }

  return paramsOut->front().get_sd();
}


/*-------------- SBrms -----------------------------*/
CSAPI_PLUGINS_BRIEF SegmentData * 
SBrms(Plugins * plugins, SOUNDfile *sfile, 
		double from=0.0, double to=FLT_MAX, 
		int fromSb=0, int toSb=575) {

  // calculate normalised subband energies
  Module * m = plugins->GetModule(string("SBrms"));
  if (m==NULL) {
    return NULL;
  }

  // get default values
  list<ModuleParam> * paramsIn = m->defaultValues();
  if (paramsIn->size() == 0) {
    return NULL;
  }

  // set parameters
  list<ModuleParam>::iterator iter;
  iter=paramsIn->begin();
  (*iter).set(sfile); // set the Soundfile ptr.
  ++iter; (*iter).set(from);
  ++iter; (*iter).set(to);
  ++iter; (*iter).set(fromSb);
  ++iter; (*iter).set(toSb);

  // suggest and check parameters
  m->suggestValues(paramsIn);

  // execute module
  list<ModuleParam> * paramsOut = m->apply(paramsIn);
  if (paramsOut->size() == 0) {
    return NULL;
  }

  return paramsOut->front().get_sd();
}

/*-------------- SBscf -----------------------------*/
CSAPI_PLUGINS_BRIEF SegmentData * 
SBscf(Plugins * plugins, SOUNDfile *sfile, 
		double from=0.0, double to=FLT_MAX, 
		int fromSb=0, int toSb=575) {

  // calculate normalised subband energies
  Module * m = plugins->GetModule(string("SBscf"));
  if (m==NULL) {
    return NULL;
  }

  // get default values
  list<ModuleParam> * paramsIn = m->defaultValues();
  if (paramsIn->size() == 0) {
    return NULL;
  }

  // set parameters
  list<ModuleParam>::iterator iter;
  iter=paramsIn->begin();
  (*iter).set(sfile); // set the Soundfile ptr.
  ++iter; (*iter).set(from);
  ++iter; (*iter).set(to);
  ++iter; (*iter).set(fromSb);
  ++iter; (*iter).set(toSb);

  // suggest and check parameters
  m->suggestValues(paramsIn);

  // execute module
  list<ModuleParam> * paramsOut = m->apply(paramsIn);
  if (paramsOut->size() == 0) {
    return NULL;
  }

  return paramsOut->front().get_sd();
}

/*-------------- SBvalue -----------------------------*/
CSAPI_PLUGINS_BRIEF SegmentData * 
SBvalue(Plugins * plugins, SOUNDfile *sfile, 
		double from=0.0, double to=FLT_MAX, 
		int fromSb=0, int toSb=575) {

  // calculate normalised subband energies
  Module * m = plugins->GetModule(string("SBvalue"));
  if (m==NULL) {
    return NULL;
  }

  // get default values
  list<ModuleParam> * paramsIn = m->defaultValues();
  if (paramsIn->size() == 0) {
    return NULL;
  }

  // set parameters
  list<ModuleParam>::iterator iter;
  iter=paramsIn->begin();
  (*iter).set(sfile); // set the Soundfile ptr.
  ++iter; (*iter).set(from);
  ++iter; (*iter).set(to);
  ++iter; (*iter).set(fromSb);
  ++iter; (*iter).set(toSb);

  // suggest and check parameters
  m->suggestValues(paramsIn);

  // execute module
  list<ModuleParam> * paramsOut = m->apply(paramsIn);
  if (paramsOut->size() == 0) {
    return NULL;
  }

  return paramsOut->front().get_sd();
}

/*-------------- SBvaluemean -----------------------------*/
CSAPI_PLUGINS_BRIEF SegmentData * 
SBvaluemean(Plugins * plugins, SOUNDfile *sfile, 
		double from=0.0, double to=FLT_MAX, 
		int fromSb=0, int toSb=575) {

  // calculate normalised subband energies
  Module * m = plugins->GetModule(string("SBvaluemean"));
  if (m==NULL) {
    return NULL;
  }

  // get default values
  list<ModuleParam> * paramsIn = m->defaultValues();
  if (paramsIn->size() == 0) {
    return NULL;
  }

  // set parameters
  list<ModuleParam>::iterator iter;
  iter=paramsIn->begin();
  (*iter).set(sfile); // set the Soundfile ptr.
  ++iter; (*iter).set(from);
  ++iter; (*iter).set(to);
  ++iter; (*iter).set(fromSb);
  ++iter; (*iter).set(toSb);

  // suggest and check parameters
  m->suggestValues(paramsIn);

  // execute module
  list<ModuleParam> * paramsOut = m->apply(paramsIn);
  if (paramsOut->size() == 0) {
    return NULL;
  }

  return paramsOut->front().get_sd();
}

/*-------------- SBvaluerms -----------------------------*/
CSAPI_PLUGINS_BRIEF SegmentData * 
SBvaluerms(Plugins * plugins, SOUNDfile *sfile, 
		double from=0.0, double to=FLT_MAX, 
		int fromSb=0, int toSb=575) {

  // calculate normalised subband energies
  Module * m = plugins->GetModule(string("SBvaluerms"));
  if (m==NULL) {
    return NULL;
  }

  // get default values
  list<ModuleParam> * paramsIn = m->defaultValues();
  if (paramsIn->size() == 0) {
    return NULL;
  }

  // set parameters
  list<ModuleParam>::iterator iter;
  iter=paramsIn->begin();
  (*iter).set(sfile); // set the Soundfile ptr.
  ++iter; (*iter).set(from);
  ++iter; (*iter).set(to);
  ++iter; (*iter).set(fromSb);
  ++iter; (*iter).set(toSb);

  // suggest and check parameters
  m->suggestValues(paramsIn);

  // execute module
  list<ModuleParam> * paramsOut = m->apply(paramsIn);
  if (paramsOut->size() == 0) {
    return NULL;
  }

  return paramsOut->front().get_sd();
}

/*-------------- SBvaluenrj -----------------------------*/
CSAPI_PLUGINS_BRIEF SegmentData * 
SBvaluenrj(Plugins * plugins, SOUNDfile *sfile, 
		double from=0.0, double to=FLT_MAX, 
		int fromSb=0, int toSb=575) {

  // calculate normalised subband energies
  Module * m = plugins->GetModule(string("SBvaluenrj"));
  if (m==NULL) {
    return NULL;
  }

  // get default values
  list<ModuleParam> * paramsIn = m->defaultValues();
  if (paramsIn->size() == 0) {
    return NULL;
  }

  // set parameters
  list<ModuleParam>::iterator iter;
  iter=paramsIn->begin();
  (*iter).set(sfile); // set the Soundfile ptr.
  ++iter; (*iter).set(from);
  ++iter; (*iter).set(to);
  ++iter; (*iter).set(fromSb);
  ++iter; (*iter).set(toSb);

  // suggest and check parameters
  m->suggestValues(paramsIn);

  // execute module
  list<ModuleParam> * paramsOut = m->apply(paramsIn);
  if (paramsOut->size() == 0) {
    return NULL;
  }

  return paramsOut->front().get_sd();
}

CSAPI_PLUGINS_BRIEF SegmentData * 
BandNrjRatio(Plugins * plugins, SOUNDfile *sfile, 
	     double from = 0.0, double to = FLT_MAX, 
	     int I = 3, int w_nb = 0) {

  // calculate normalised subband energies
  Module * m = plugins->GetModule(string("BandNrjRatio"));
  if (m==NULL) {
    return NULL;
  }

  // get default values
  list<ModuleParam> * paramsIn = m->defaultValues();
  if (paramsIn->size() == 0) {
    return NULL;
  }

  // set parameters
  list<ModuleParam>::iterator iter;
  iter=paramsIn->begin();
  (*iter).set(sfile); // set the Soundfile ptr.
  ++iter; (*iter).set(from);
  ++iter; (*iter).set(to);
  ++iter; (*iter).set(I);
  ++iter; (*iter).set(w_nb);

  // suggest and check parameters
  m->suggestValues(paramsIn);

  // execute module
  list<ModuleParam> * paramsOut = m->apply(paramsIn);
  if (paramsOut->size() == 0) {
    return NULL;
  }

  return paramsOut->front().get_sd();
}

  
CSAPI_PLUGINS_BRIEF Trio 
bandwidth(Plugins * plugins, SOUNDfile *sfile, 
	  double from = 0.0 , double to = FLT_MAX, 
	  double Ts = 0.001) {

  Trio ret;
  ret.uno =0;
  ret.dos =0;
  ret.tres = 0;
  // calculate normalised subband energies
  Module * m = plugins->GetModule(string("bandwidth"));
  if (m==NULL) {
    return ret;
  }

  // get default values
  list<ModuleParam> * paramsIn = m->defaultValues();
  if (paramsIn->size() == 0) {
    return ret;
  }

  // set parameters
  list<ModuleParam>::iterator iter;
  iter=paramsIn->begin();
  (*iter).set(sfile); // set the Soundfile ptr.
  ++iter; (*iter).set(from);
  ++iter; (*iter).set(to);
  ++iter; (*iter).set(Ts);

  // suggest and check parameters
  m->suggestValues(paramsIn);

  // execute module
  list<ModuleParam> * paramsOut = m->apply(paramsIn);
  if (paramsOut->size() == 0) {
    return ret;
  }

  iter=paramsOut->begin();
  ret.uno = (*iter).get_sd();
  ++iter; ret.dos = (*iter).get_sd();
  ++iter; ret.tres = (*iter).get_sd();
  return ret;
}

CSAPI_PLUGINS_BRIEF SegmentData *
sigSb(Plugins * plugins, SOUNDfile *sfile, 
      double from = 0.0 , double to = FLT_MAX, 
      double Ts = 0.001) {

  // calculate normalised subband energies
  Module * m = plugins->GetModule(string("sigsb"));
  if (m==NULL) {
    return NULL;
  }

  // get default values
  list<ModuleParam> * paramsIn = m->defaultValues();
  if (paramsIn->size() == 0) {
    return NULL;
  }

  // set parameters
  list<ModuleParam>::iterator iter;
  iter=paramsIn->begin();
  (*iter).set(sfile); // set the Soundfile ptr.
  ++iter; (*iter).set(from);
  ++iter; (*iter).set(to);
  ++iter; (*iter).set(Ts);

  // suggest and check parameters
  m->suggestValues(paramsIn);

  // execute module
  list<ModuleParam> * paramsOut = m->apply(paramsIn);
  if (paramsOut->size() == 0) {
    return NULL;
  }

  return paramsOut->front().get_sd();
}

CSAPI_PLUGINS_BRIEF SegmentData * 
centralMoment(Plugins * plugins, SOUNDfile *sfile, 
	      double from = 0.0, double to = FLT_MAX, 
	      int fromSb = 0, int toSb = 31,
	      double duration = 1.0, int k = 2) {

  // calculate normalised subband energies
  Module * m = plugins->GetModule(string("centralMoment"));
  if (m==NULL) {
    return NULL;
  }

  // get default values
  list<ModuleParam> * paramsIn = m->defaultValues();
  if (paramsIn->size() == 0) {
    return NULL;
  }

  // set parameters
  list<ModuleParam>::iterator iter;
  iter=paramsIn->begin();
  (*iter).set(sfile); // set the Soundfile ptr.
  ++iter; (*iter).set(from);
  ++iter; (*iter).set(to);
  ++iter; (*iter).set(fromSb);
  ++iter; (*iter).set(toSb);
  ++iter; (*iter).set(duration);
  ++iter; (*iter).set(k);

  // suggest and check parameters
  m->suggestValues(paramsIn);

  // execute module
  list<ModuleParam> * paramsOut = m->apply(paramsIn);
  if (paramsOut->size() == 0) {
    return NULL;
  }

  return paramsOut->front().get_sd();
}

CSAPI_PLUGINS_BRIEF SegmentData * 
signalnrj(Plugins * plugins, SOUNDfile *sfile, 
	  double from = 0.0, double to = FLT_MAX,
	  int w_nb = 0) {

  // calculate normalised subband energies
  Module * m = plugins->GetModule(string("signalnrj"));
  if (m==NULL) {
    return NULL;
  }

  // get default values
  list<ModuleParam> * paramsIn = m->defaultValues();
  if (paramsIn->size() == 0) {
    return NULL;
  }

  // set parameters
  list<ModuleParam>::iterator iter;
  iter=paramsIn->begin();
  (*iter).set(sfile); // set the Soundfile ptr.
  ++iter; (*iter).set(from);
  ++iter; (*iter).set(to);
  ++iter; (*iter).set(w_nb);

  // suggest and check parameters
  m->suggestValues(paramsIn);

  // execute module
  list<ModuleParam> * paramsOut = m->apply(paramsIn);
  if (paramsOut->size() == 0) {
    return NULL;
  }

  return paramsOut->front().get_sd();
}

CSAPI_PLUGINS_BRIEF SegmentData * 
bandNrj(Plugins * plugins, SOUNDfile *sfile, 
	double from = 0.0, double to = FLT_MAX,
	int fromSb=0, int toSb=575,
	int w_nb = 0) {

  // calculate normalised subband energies
  Module * m = plugins->GetModule(string("bandnrj"));
  if (m==NULL) {
    return NULL;
  }

  // get default values
  list<ModuleParam> * paramsIn = m->defaultValues();
  if (paramsIn->size() == 0) {
    return NULL;
  }

  // set parameters
  list<ModuleParam>::iterator iter;
  iter=paramsIn->begin();
  (*iter).set(sfile); // set the Soundfile ptr.
  ++iter; (*iter).set(from);
  ++iter; (*iter).set(to);
  ++iter; (*iter).set(fromSb);
  ++iter; (*iter).set(toSb);
  ++iter; (*iter).set(w_nb);

  // suggest and check parameters
  m->suggestValues(paramsIn);

  // execute module
  list<ModuleParam> * paramsOut = m->apply(paramsIn);
  if (paramsOut->size() == 0) {
    return NULL;
  }

  return paramsOut->front().get_sd();
}

CSAPI_PLUGINS_BRIEF SegmentData * 
spectralFlux(Plugins * plugins, SOUNDfile *sfile, 
	     double from = 0.0, double to = FLT_MAX) {

  // calculate normalised subband energies
  Module * m = plugins->GetModule(string("spectralFlux"));
  if (m==NULL) {
    return NULL;
  }

  // get default values
  list<ModuleParam> * paramsIn = m->defaultValues();
  if (paramsIn->size() == 0) {
    return NULL;
  }

  // set parameters
  list<ModuleParam>::iterator iter;
  iter=paramsIn->begin();
  (*iter).set(sfile); // set the Soundfile ptr.
  ++iter; (*iter).set(from);
  ++iter; (*iter).set(to);

  // suggest and check parameters
  m->suggestValues(paramsIn);

  // execute module
  list<ModuleParam> * paramsOut = m->apply(paramsIn);
  if (paramsOut->size() == 0) {
    return NULL;
  }

  return paramsOut->front().get_sd();
}

CSAPI_PLUGINS_BRIEF SegmentData * 
pauseRate(Plugins * plugins, SegmentData *sd, 
	  double from = 0.0, double to = FLT_MAX, 
	  double Ts = 0.001, double duration = 1.0) {

  // calculate normalised subband energies
  Module * m = plugins->GetModule(string("pauseRate"));
  if (m==NULL) {
    return NULL;
  }

  // get default values
  list<ModuleParam> * paramsIn = m->defaultValues();
  if (paramsIn->size() == 0) {
    return NULL;
  }

  // set parameters
  list<ModuleParam>::iterator iter;
  iter=paramsIn->begin();
  (*iter).set(sd); // set the SegmentData
  ++iter; (*iter).set(from);
  ++iter; (*iter).set(to);
  ++iter; (*iter).set(Ts);
  ++iter; (*iter).set(duration);

  // suggest and check parameters
  m->suggestValues(paramsIn);

  // execute module
  list<ModuleParam> * paramsOut = m->apply(paramsIn);
  if (paramsOut->size() == 0) {
    return NULL;
  }

  return paramsOut->front().get_sd();
}

CSAPI_PLUGINS_BRIEF SegmentData * 
rollOff(Plugins * plugins, SOUNDfile *sfile, 
	double from = 0.0, double to = FLT_MAX) {

  // calculate normalised subband energies
  Module * m = plugins->GetModule(string("rollOff"));
  if (m==NULL) {
    return NULL;
  }

  // get default values
  list<ModuleParam> * paramsIn = m->defaultValues();
  if (paramsIn->size() == 0) {
    return NULL;
  }

  // set parameters
  list<ModuleParam>::iterator iter;
  iter=paramsIn->begin();
  (*iter).set(sfile); // set the Soundfile ptr.
  ++iter; (*iter).set(from);
  ++iter; (*iter).set(to);

  // suggest and check parameters
  m->suggestValues(paramsIn);

  // execute module
  list<ModuleParam> * paramsOut = m->apply(paramsIn);
  if (paramsOut->size() == 0) {
    return NULL;
  }

  return paramsOut->front().get_sd();
}

CSAPI_PLUGINS_BRIEF Trio
sigBdwidth(Plugins * plugins, SegmentData *sd, 
	   double from = 0.0, double to = FLT_MAX, 
	   double Ts = 0.001) {
  Trio ret;
  ret.uno = 0;
  ret.dos = 0;
  ret.tres = 0;
  // calculate normalised subband energies
  Module * m = plugins->GetModule(string("sigBdwidth"));
  if (m==NULL) {
    return ret;
  }

  // get default values
  list<ModuleParam> * paramsIn = m->defaultValues();
  if (paramsIn->size() == 0) {
    return ret;
  }

  // set parameters
  list<ModuleParam>::iterator iter;
  iter=paramsIn->begin();
  (*iter).set(sd); // set the Segment data
  ++iter; (*iter).set(from);
  ++iter; (*iter).set(to);
  ++iter; (*iter).set(Ts);

  // suggest and check parameters
  m->suggestValues(paramsIn);

  // execute module
  list<ModuleParam> * paramsOut = m->apply(paramsIn);
  if (paramsOut->size() == 0) {
    return ret;
  }


  iter=paramsOut->begin();
  ret.uno = (*iter).get_sd();
  ++iter; ret.dos = (*iter).get_sd();
  ++iter; ret.tres = (*iter).get_sd();
  return ret;
}

CSAPI_PLUGINS_BRIEF SegmentData * 
signalMagnitude(Plugins * plugins, SOUNDfile *sfile, 
		double from = 0.0, double to = FLT_MAX, 
		int w_nb = 0) {

  // calculate normalised subband energies
  Module * m = plugins->GetModule(string("signalMagnitude"));
  if (m==NULL) {
    return NULL;
  }

  // get default values
  list<ModuleParam> * paramsIn = m->defaultValues();
  if (paramsIn->size() == 0) {
    return NULL;
  }

  // set parameters
  list<ModuleParam>::iterator iter;
  iter=paramsIn->begin();
  (*iter).set(sfile); // set the Soundfile ptr.
  ++iter; (*iter).set(from);
  ++iter; (*iter).set(to);
  ++iter; (*iter).set(w_nb);

  // suggest and check parameters
  m->suggestValues(paramsIn);

  // execute module
  list<ModuleParam> * paramsOut = m->apply(paramsIn);
  if (paramsOut->size() == 0) {
    return NULL;
  }

  return paramsOut->front().get_sd();
}

