/*
    MPEG Maaate: An Australian MPEG audio analysis toolkit
    Copyright (C) 2000 Commonwealth Scientific and Industrial Research Organisation
    (CSIRO), Australia.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "SOUNDfile.H"

/* constructors and destructor */
CSAPI_TIER1 
SOUNDfile * maaateP_new_soundfile(char * filestr) {
  return new SOUNDfile(filestr);
}

CSAPI_TIER1 void maaateP_destroy(SOUNDfile * sf) {
  delete sf;
}

/* about this file */
CSAPI_TIER1 char * maaateP_get_filename(SOUNDfile * sf) {
  return (char *) (sf->file()).c_str();
}

CSAPI_TIER1 Format maaateP_get_filetype(SOUNDfile * sf) {
  return sf->file_type();
}

CSAPI_TIER1 int maaateP_is_stereo(SOUNDfile * sf) {
  return (int) sf->is_stereo();
}

CSAPI_TIER1 int maaateP_channels(SOUNDfile * sf) {
  return sf->channels();
}

CSAPI_TIER1 double maaateP_samplingrate(SOUNDfile * sf) {
  return sf->sampling_rate();
}

/* time functions */
CSAPI_TIER1 float maaateP_at_time(SOUNDfile * sf)  {
  return sf->at_time();
}

CSAPI_TIER1 float maaateP_file_duration(SOUNDfile * sf)  {
  return sf->file_duration();
}

CSAPI_TIER1 float maaateP_sample_duration(SOUNDfile * sf, Resolution res) {
  return sf->sample_duration(res);
}

CSAPI_TIER1 int maaateP_seek_time(SOUNDfile * sf, float t )  {
  return sf->seek_time(t);
}

CSAPI_TIER1 float maaateP_window_duration(SOUNDfile * sf) {
  return sf->window_duration();
}


/* window functions */
CSAPI_TIER1 long maaateP_at_window(SOUNDfile * sf) {
  return sf->at_window();
}

CSAPI_TIER1 long maaateP_file_window_number(SOUNDfile * sf) {
  return sf->file_window_number();
}

CSAPI_TIER1 unsigned int maaateP_timeticks(SOUNDfile * sf, Resolution res) {
  return sf->timeticks(res);
}

CSAPI_TIER1 int maaateP_seek_window(SOUNDfile * sf, long w_nb )  {
  return sf->seek_window(w_nb);
}

/* matching functions */
CSAPI_TIER1 long maaateP_time2window(SOUNDfile * sf, float t ) {
  return sf->time2window(t);
}

CSAPI_TIER1 float maaateP_window2time(SOUNDfile * sf, long w_nb ) {
  return sf->window2time(w_nb);
}


/* extract and skip */
CSAPI_TIER1 int maaateP_next_window(SOUNDfile * sf, Resolution res) {
  return sf->next_window(res);
}

CSAPI_TIER1 int maaateP_skip_window(SOUNDfile * sf) {
  return sf->skip_window();
}


/* access function */
CSAPI_TIER1 double maaateP_freq_value(SOUNDfile * sf, unsigned int ch, 
			unsigned int sb, unsigned int nb, 
			Resolution res)
{
  return sf->freq_value(ch,sb,nb,res);
}

CSAPI_TIER1 unsigned int maaateP_nb_subbands(SOUNDfile * sf, Resolution res) {
  return sf->nb_subbands(res);
}

/* Analyse functions */

//time resolution = 1 subband sample
  
CSAPI_TIER1 double maaateP_freqvalue_st_mean(SOUNDfile * sf, unsigned int sb, 
			       unsigned int nb, Resolution res)
{
  return sf->freqvalue_st_mean(sb,nb,res);
}

CSAPI_TIER1 double maaateP_freqvalue_st_rms(SOUNDfile * sf, unsigned int sb, 
			      unsigned int nb, Resolution res)
{
  return sf->freqvalue_st_rms(sb,nb,res);
}

CSAPI_TIER1 double maaateP_normalised_sb_nrj(SOUNDfile * sf, unsigned int sb, 
			       unsigned int nb, Resolution res)
{
  return sf->normalised_sb_nrj(sb,nb,res);
}

// time resolution = 1 window
  
CSAPI_TIER1 double maaateP_subband_mean (SOUNDfile * sf, unsigned int sb, Resolution res)
{
  return sf->subband_mean(sb,res);
}

CSAPI_TIER1 double maaateP_subband_rms (SOUNDfile * sf, unsigned int sb, Resolution res)
{
  return sf->subband_rms(sb,res);
}

CSAPI_TIER1 double maaateP_subband_nrj (SOUNDfile * sf, unsigned int sb, Resolution res)
{
  return sf->subband_nrj(sb,res);
}

/* decoder*/
CSAPI_TIER1 long maaateP_decode (SOUNDfile * sf, short * buffer, long windows, Channels ch) {
  return sf->decode(buffer,windows,ch);
}

/* File format specific pointer access */
CSAPI_TIER1 AllFormat * maaateP_format_access(SOUNDfile * sf)
{
  return sf->format;
}
