/*
    MPEG Maaate: An Australian MPEG audio analysis toolkit
    Copyright (C) 2000 Commonwealth Scientific and Industrial Research Organisation
    (CSIRO), Australia.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <cmath>
#include "module.H"

CSAPI_TIER2 Module *
maaateA_new_module(ModuleInitFunc      initF,
		   ModuleDefaultFunc   defaultF,
		   ModuleSuggestValues suggestF,
		   ModuleApplyFunc     applyF,
		   ModuleResetFunc     resetF,
		   ModuleDestroyFunc   destroyF)
{
  return new Module(initF,
		    defaultF,
		    suggestF,
		    applyF,
		    resetF,
		    destroyF);
}

CSAPI_TIER2 void
maaateA_destroy_module(Module * m)
{
  m->~Module();
}

CSAPI_TIER2 int
maaateA_sane_module   (Module * m)
{
  return m->sane();
}

CSAPI_TIER2 const char *
maaateA_name_module   (Module * m)
{
  return (m->name()).c_str();
}

CSAPI_TIER2 const char *
maaateA_desc_module   (Module * m)
{
  return (m->desc()).c_str();
}

CSAPI_TIER2 const char *
maaateA_author_module (Module * m)
{
  return (m->author()).c_str();
}

CSAPI_TIER2 const char *
maaateA_copyright_module(Module * m)
{
  return (m->copyright()).c_str();
}

CSAPI_TIER2 const char *
maaateA_url_module    (Module * m)
{
  return (m->url()).c_str();
}

CSAPI_TIER2 ModuleParamSpecList *
maaateA_inputSpecs (Module * m)
{
  return (m->inputSpecs());
}

CSAPI_TIER2 ModuleParamSpecList *
maaateA_outputSpecs(Module * m)
{
  return (m->outputSpecs());
}

CSAPI_TIER2 ModuleParamList *
maaateA_default_values (Module * m)
{
  return (m->defaultValues());
}

CSAPI_TIER2 void
maaateA_suggest_values (Module * m,
			ModuleParamList * paramsIn)
{
  m->suggestValues(paramsIn);
}

CSAPI_TIER2 ModuleParamList *
maaateA_apply (Module * m, ModuleParamList * paramsIn)
{
  return (m->apply(paramsIn));
}

CSAPI_TIER2 void
maaateA_reset (Module * m)
{
  m->reset();
}

CSAPI_TIER2 int
maaateA_check_constraints (Module * m, ModuleParamList * paramsIn)
{
  return m->checkConstraints(paramsIn);
}

/* wrapper functions for MaaateConstraint */
CSAPI_TIER2 ModuleParamConstraintList * 
maaateA_MaaateConstraint_getConstraints(MaaateConstraint * mc)
{
  return mc->getConstraints();
}

CSAPI_TIER2 unsigned int 
maaateA_MaaateConstraint_withinConstraints (MaaateConstraint * mc, ModuleParam * value)
{
  return (unsigned int)(mc->withinConstraints(*value));
}

CSAPI_TIER2 ModuleParam * 
maaateA_MaaateConstraint_closestConstraintValue (MaaateConstraint * mc, ModuleParam * value)
{
  return mc->closestConstraintValue(*value);
}


/* wrapper functions for ModuleParamSpec */
CSAPI_TIER2 const char * 
maaateA_ModuleParamSpec_name (ModuleParamSpec * mps) {
  return (mps->name()).c_str();
}
  
CSAPI_TIER2 const char * 
maaateA_ModuleParamSpec_desc (ModuleParamSpec * mps) {
  return (mps->desc()).c_str();
}

CSAPI_TIER2 MaaateType 
maaateA_ModuleParamSpec_type (ModuleParamSpec * mps) {
  return mps->type();
}

CSAPI_TIER2 ModuleParam *
maaateA_ModuleParamSpec_defaultvalue (ModuleParamSpec * mps) {
  return mps->defaultValue();
}

CSAPI_TIER2 MaaateConstraint * 
maaateA_ModuleParamSpec_constraint (ModuleParamSpec * mps) {
 return mps->constraint();
}

CSAPI_TIER2 MaaateType 
maaateA_ModuleParam_getType(ModuleParam * mp) {
  return mp->getType();
}

CSAPI_TIER2 void
maaateA_ModuleParam_setSOUNDfile (ModuleParam * mp, SOUNDfile * sf)
{
  mp->set(sf);
}

CSAPI_TIER2 void
maaateA_ModuleParam_setSegmentTable (ModuleParam * mp, SegmentTable * st)
{
  mp->set(st);
}

CSAPI_TIER2 void
maaateA_ModuleParam_setSegmentData (ModuleParam * mp, SegmentData * sd)
{
  mp->set(sd);
}

CSAPI_TIER2 void
maaateA_ModuleParam_setBool (ModuleParam * mp, int b)
{
  mp->set((bool) b);
}

CSAPI_TIER2 void
maaateA_ModuleParam_setInt  (ModuleParam * mp, int i)
{
  mp->set(i);
}

CSAPI_TIER2 void
maaateA_ModuleParam_setReal (ModuleParam * mp, double r)
{
  mp->set(r);
}

CSAPI_TIER2 void
maaateA_ModuleParam_setString (ModuleParam * mp, char * s)
{
  mp->set(s);
}

CSAPI_TIER2 SOUNDfile *
maaateA_ModuleParam_getSOUNDfile (ModuleParam * mp)
{
  return mp->get_sf();
}

CSAPI_TIER2 SegmentTable *
maaateA_ModuleParam_getSegmentTable (ModuleParam * mp)
{
  return mp->get_st();
}

CSAPI_TIER2 SegmentData *
maaateA_ModuleParam_getSegmentData (ModuleParam * mp)
{
  return mp->get_sd();
}

CSAPI_TIER2 int
maaateA_ModuleParam_getBool (ModuleParam * mp)
{
  return mp->get_b();
}

CSAPI_TIER2 int
maaateA_ModuleParam_getInt  (ModuleParam * mp)
{
  return mp->get_i();
}

CSAPI_TIER2 double
maaateA_ModuleParam_getReal (ModuleParam * mp)
{
  return mp->get_r();
}

CSAPI_TIER2 char *
maaateA_ModuleParam_getString (ModuleParam * mp)
{
  return mp->get_s();
}


/* list handling */
CSAPI_TIER2 ModuleParamConstraint * 
maaateA_ModuleParamConstraintList_nth (ModuleParamConstraintList * plist,
				       unsigned int n)
{
  list<ModuleParamConstraint>::iterator iter;
  iter=plist->begin();
  for (unsigned int i=0; i<n; i++) {
    ++iter;
  }
  return &(*iter);

}

CSAPI_TIER2 unsigned int 
maaateA_ModuleParamConstraintList_length (ModuleParamConstraintList * plist)
{
  return plist->size();
}

CSAPI_TIER2 ModuleParamSpec * 
maaateA_ModuleParamSpecList_nth (ModuleParamSpecList * plist,
				 unsigned int n)
{
  list<ModuleParamSpec>::iterator iter;
  iter=plist->begin();
  for (unsigned int i=0; i<n; i++) {
    ++iter;
  }
  return &(*iter);

}

CSAPI_TIER2 unsigned int 
maaateA_ModuleParamSpecList_length (ModuleParamSpecList * plist)
{
  return plist->size();
}

CSAPI_TIER2 void
maaateA_ModuleParamList_print (ModuleParamList * plist) 
{
  list<ModuleParam>::iterator iter;
  for (iter=(plist->begin()); iter!=(plist->end());iter++) {
    cout << (*iter) << endl;
  }
  return;
  
}

CSAPI_TIER2 ModuleParam * 
maaateA_ModuleParamList_nth (ModuleParamList * plist,
			     unsigned int n)
{
  list<ModuleParam>::iterator iter;
  iter=plist->begin();
  for (unsigned int i=0; i<n; i++) {
    ++iter;
  }
  return &(*iter);

}

CSAPI_TIER2 unsigned int 
maaateA_ModuleParamList_length (ModuleParamList * plist)
{
  return plist->size();
}

CSAPI_TIER2 Module * 
maaateA_ModuleList_nth (ModuleList * mlist,
			unsigned int n)
{
  list<Module>::iterator iter;
  iter=mlist->begin();
  for (unsigned int i=0; i<n; i++) {
    ++iter;
  }
  return &(*iter);

}

CSAPI_TIER2 unsigned int 
maaateA_ModuleList_length (ModuleList * mlist)
{
  return mlist->size();
}
