/*
    MPEG Maaate: An Australian MPEG audio analysis toolkit
    Copyright (C) 2000 Commonwealth Scientific and Industrial Research Organisation
    (CSIRO), Australia.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <cmath>
#include "segmentData.H"

CSAPI_TIER2 SegmentData * 
maaateA_sd_new(double t1, double t2, int col, int row, 
			int i, char f, double conf) 
{
  return new SegmentData(t1, t2, col, row, i, f, conf);
}

CSAPI_TIER2 SegmentData * 
maaateA_sd_new_default(void)
{
  return new SegmentData();
}

CSAPI_TIER2 void 
maaateA_sd_delete(SegmentData *sd)
{
  delete sd;
}

CSAPI_TIER2 int           
maaateA_sd_starts_before(SegmentData *sd1, SegmentData *sd2)
{
  return ((*sd1) < (*sd2));
}

CSAPI_TIER2 int
maaateA_sd_ends_after(SegmentData *sd1, SegmentData *sd2)
{
  return  ((*sd1) > (*sd2));
}

CSAPI_TIER2 void
maaateA_sd_print(SegmentData * sd)
{
  cout << *sd;
}

CSAPI_TIER2 SegmentData * 
maaateA_sd_copy(SegmentData *sd)
{
  return new SegmentData(*sd);
}

CSAPI_TIER2 SegmentData * 
maaateA_sd_partial_copy   (SegmentData *sd, int st_row, int end_row)
{
  return new SegmentData(*sd, st_row, end_row);
}

CSAPI_TIER2 double
maaateA_sd_element(SegmentData *sd, int c, int r)
{
  return (*sd)(c,r);
}

CSAPI_TIER2 double 
maaateA_sd_get_start(SegmentData * sd)
{
  return sd->start();
}

CSAPI_TIER2 double 
maaateA_sd_get_end(SegmentData * sd)
{
  return sd->end();
}

CSAPI_TIER2 double 
maaateA_sd_get_duration(SegmentData * sd)
{
  return sd->duration();
}

CSAPI_TIER2 double 
maaateA_sd_get_resolution(SegmentData * sd)
{
  return sd->resolution();
}

CSAPI_TIER2 double
maaateA_sd_get_filled(SegmentData * sd)
{
  return sd->filled();
}

CSAPI_TIER2 double
maaateA_sd_get_filledUntil(SegmentData * sd)
{
  return sd->filledUntil();
}

CSAPI_TIER2 int
maaateA_sd_col_filled(SegmentData * sd)
{
  return sd->col_filled();
}

CSAPI_TIER2 int
maaateA_sd_no_columns(SegmentData *sd)
{
  return sd->no_columns();
}

CSAPI_TIER2 int
maaateA_sd_no_rows(SegmentData *sd)
{
  return sd->no_rows();
}

CSAPI_TIER2 int
maaateA_sd_get_ident(SegmentData * sd)
{
  return sd->ident();
}

CSAPI_TIER2 char
maaateA_sd_get_uflag(SegmentData * sd)
{
  return sd->uflag();
}

CSAPI_TIER2 double
maaateA_sd_get_confidence(SegmentData *sd)
{
  return sd->confidence();
}

CSAPI_TIER2 int
maaateA_sd_time2col (SegmentData *sd, double time)
{
  return sd->time2col(time);
}

CSAPI_TIER2 double
maaateA_sd_sum_part (SegmentData * sd, int start, int end, 
		     int startrow, int endrow)
{
  return sd->sum(start, end, startrow, endrow);
}

CSAPI_TIER2 double
maaateA_sd_min_part (SegmentData * sd, double start, double end, 
		     int startrow, int endrow)
{
  return sd->smin(start, end, startrow, endrow);
}

CSAPI_TIER2 double
maaateA_sd_max_part (SegmentData * sd, double start, double end, 
		     int startrow, int endrow)
{
  return sd->smax(start, end, startrow, endrow);
}

CSAPI_TIER2 double
maaateA_sd_avg_part (SegmentData * sd, int start, int end, 
		     int startrow, int endrow)
{
  return sd->avg(start, end, startrow, endrow);
}

CSAPI_TIER2 double
maaateA_sd_sum(SegmentData * sd)
{
  return sd->sum();
}

CSAPI_TIER2 double
maaateA_sd_min(SegmentData * sd, double downto )
{
  return sd->smin(downto);
}

CSAPI_TIER2 double
maaateA_sd_max(SegmentData * sd, double upto)
{
  return sd->smax(upto);
}

CSAPI_TIER2 double
maaateA_sd_avg(SegmentData * sd)
{
  return sd->avg();
}

CSAPI_TIER2 int
maaateA_sd_normalise(SegmentData * sd, double downto, double upto)
{
  return sd->normalise(downto, upto);
}
  
CSAPI_TIER2 void
maaateA_sd_clear_sum(SegmentData * sd)
{
  sd->clear_sum();
}

CSAPI_TIER2 void
maaateA_sd_clear_min(SegmentData * sd)
{
  sd->clear_min();
}

CSAPI_TIER2 void
maaateA_sd_clear_max(SegmentData * sd)
{
  sd->clear_max();
}

CSAPI_TIER2 void 
maaateA_sd_clear_all(SegmentData * sd)
{
  sd->clear_all();
}

CSAPI_TIER2 void 
maaateA_sd_initData (SegmentData * sd, double val)
{
  sd->initData(val);
}
