/*
    MPEG Maaate: An Australian MPEG audio analysis toolkit
    Copyright (C) 2000 Commonwealth Scientific and Industrial Research Organisation
    (CSIRO), Australia.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "module.H"

// copy constructor (XXX is it necessary anyway?)
CSAPI_TIER2 
ModuleParam::ModuleParam (const ModuleParam& mp) {
  type = mp.getType();
  switch(type) {
  case MAAATE_TYPE_SOUNDFILE:
    sf = mp.get_sf(); break;
  case MAAATE_TYPE_SEGMENTTABLE:
    st = mp.get_st(); break;
  case MAAATE_TYPE_SEGMENTDATA:
    sd = mp.sd; break;
  case MAAATE_TYPE_BOOL:
    b = mp.get_b(); break;
  case MAAATE_TYPE_INT:
    i = mp.get_i(); break;
  case MAAATE_TYPE_REAL:
    r = mp.get_r(); break;
  case MAAATE_TYPE_STRING:
    s = strdup(mp.get_s()); break;
  }
}
  
// overload friend operator <<
// main usage: debugging
CSAPI_TIER2 ostream&
operator<<(ostream& os, ModuleParam const & mp) {
  switch (mp.getType()) {
  case MAAATE_TYPE_SOUNDFILE:
    if (mp.get_sf() != NULL) {
      os << mp.get_sf()->file();
    }
    break;
  case MAAATE_TYPE_SEGMENTTABLE:
    if (mp.get_st() != NULL) {
      os << mp.get_st();
    }
    break;
  case MAAATE_TYPE_SEGMENTDATA:
    if (mp.get_sd() != NULL) {
      os << mp.get_sd();
    }
    break;
  case MAAATE_TYPE_BOOL:
    os << mp.get_b();
    break;
  case MAAATE_TYPE_INT:
    os << mp.get_i();
    break;
  case MAAATE_TYPE_REAL:
    os << mp.get_r();
    break;
  case MAAATE_TYPE_STRING:
    if (mp.get_s() != NULL) {
      os << mp.get_s();
    }
    break;
  }
  return os;
}

CSAPI_TIER2 bool
operator== (ModuleParam const & mp1, ModuleParam const & mp2) {
  if (mp1.getType() != mp2.getType()) return (false);

  switch(mp1.getType()) {
  case MAAATE_TYPE_SOUNDFILE:
    return (mp1.get_sf() == mp2.get_sf());
  case MAAATE_TYPE_SEGMENTTABLE:
    return (mp1.get_st() == mp2.get_st());
  case MAAATE_TYPE_SEGMENTDATA:
    return (mp1.get_sd() == mp2.get_sd());
  case MAAATE_TYPE_BOOL:
    return (mp1.get_b() == mp2.get_b());
  case MAAATE_TYPE_INT:
    return (mp1.get_i() == mp2.get_i());
  case MAAATE_TYPE_REAL:
    return (mp1.get_r() == mp2.get_r());
  case MAAATE_TYPE_STRING:
    return (!strcmp (mp1.get_s(), mp2.get_s()));
  default:
    return false;
  }

  return false;
}

CSAPI_TIER2 ModuleParam&
ModuleParam::operator= (const ModuleParam & mp) {
  if (this == &mp)
    return *this;
  type = mp.getType();
  switch(type) {
  case MAAATE_TYPE_SOUNDFILE:
    sf = mp.get_sf();
    break;
  case MAAATE_TYPE_SEGMENTTABLE:
    st = mp.get_st();
    break;
  case MAAATE_TYPE_SEGMENTDATA:
    sd = mp.sd;
    break;
  case MAAATE_TYPE_BOOL:
    b = mp.get_b();
    break;
  case MAAATE_TYPE_INT:
    i = mp.get_i();
    break;
  case MAAATE_TYPE_REAL:
    r = mp.get_r();
    break;
  case MAAATE_TYPE_STRING:
    strcpy (s, mp.get_s());
    break;
  }
  return *this;
}

CSAPI_TIER2 double
operator- (ModuleParam const & mp1, ModuleParam const & mp2) {
  if (mp1.getType() != mp2.getType()) return (false);

  switch(mp1.getType()) {
  case MAAATE_TYPE_SOUNDFILE:
    return (DBL_MAX);
  case MAAATE_TYPE_SEGMENTTABLE:
    return (DBL_MAX);
  case MAAATE_TYPE_SEGMENTDATA:
    return (DBL_MAX);
  case MAAATE_TYPE_BOOL:
    return (mp1.get_b() == mp2.get_b() ? 0.0 : DBL_MAX);
  case MAAATE_TYPE_INT:
    return ((double)(mp1.get_i() - mp2.get_i()));
  case MAAATE_TYPE_REAL:
    return (mp1.get_r() - mp2.get_r());
  case MAAATE_TYPE_STRING:
    return ((double)strcmp (mp1.get_s(), mp2.get_s()));
  default:
    return false;
  }
  return false;
}

CSAPI_TIER2 bool
operator< (ModuleParam const & mp1, ModuleParam const & mp2) {
  if (mp1.getType() != mp2.getType()) return (false);

  switch(mp1.getType()) {
  case MAAATE_TYPE_SOUNDFILE:
    return (false);
  case MAAATE_TYPE_SEGMENTTABLE:
    return (false);
  case MAAATE_TYPE_SEGMENTDATA:
    return (false);
  case MAAATE_TYPE_BOOL:
    return (mp1.get_b() != mp2.get_b());
  case MAAATE_TYPE_INT:
    return (mp1.get_i() < mp2.get_i());
  case MAAATE_TYPE_REAL:
    return (mp1.get_r() < mp2.get_r());
  case MAAATE_TYPE_STRING:
    return (strcmp (mp1.get_s(), mp2.get_s()) < 0);
  default:
    return (false);
  }
  return false;
}

CSAPI_TIER2 bool
operator> (ModuleParam const & mp1, ModuleParam const & mp2) {
  if (mp1.getType() != mp2.getType()) return (false);

  switch(mp1.getType()) {
  case MAAATE_TYPE_SOUNDFILE:
    return (false);
  case MAAATE_TYPE_SEGMENTTABLE:
    return (false);
  case MAAATE_TYPE_SEGMENTDATA:
    return (false);
  case MAAATE_TYPE_BOOL:
    return (mp1.get_b() != mp2.get_b());
  case MAAATE_TYPE_INT:
    return (mp1.get_i() > mp2.get_i());
  case MAAATE_TYPE_REAL:
    return (mp1.get_r() > mp2.get_r());
  case MAAATE_TYPE_STRING:
    return (strcmp (mp1.get_s(), mp2.get_s()) > 0);
  default:
    return false;
  }
  return false;
}

CSAPI_TIER2 bool
operator<= ( ModuleParam const & mp1, ModuleParam const & mp2) {
  return ((operator<((const ModuleParam&) mp1,mp2)) || (mp1 == mp2));
}

CSAPI_TIER2 bool
operator>= (ModuleParam const & mp1, ModuleParam const & mp2) {
  return ((mp1 > mp2) || (mp1 == mp2));
}

CSAPI_TIER2 bool
ModuleParam::isZero () {
  switch(type) {
  case MAAATE_TYPE_SOUNDFILE:
    return (sf == NULL);
  case MAAATE_TYPE_SEGMENTTABLE:
    return (st == NULL);
  case MAAATE_TYPE_SEGMENTDATA:
    return (sd == NULL);
  case MAAATE_TYPE_BOOL:
    return (!b);
  case MAAATE_TYPE_INT:
    return (i == 0);
  case MAAATE_TYPE_REAL:
    return (r == 0.0);
  case MAAATE_TYPE_STRING:
    return (!strcmp (s, ""));
  default:
    return (false);
  }
  return false;
}

CSAPI_TIER2 bool 
ModuleParam::set (SOUNDfile * spf) {
  if (type == MAAATE_TYPE_SOUNDFILE) {
    sf = spf;
    return true;
  } else {
    return false;
  }
}

CSAPI_TIER2 bool
ModuleParam::set (SegmentTable * sgt) {
  if (type == MAAATE_TYPE_SEGMENTTABLE) {
    st = sgt;
    return true;
  } else {
    return false;
  }
}

CSAPI_TIER2 bool
ModuleParam::set (SegmentData * sgd) {
  if (type == MAAATE_TYPE_SEGMENTDATA) {
    sd = sgd;
    return true;
  } else {
    return false;
  }
}

CSAPI_TIER2 bool
ModuleParam::set (bool bl) {
  if (type == MAAATE_TYPE_BOOL) {
    b = bl;
    return true;
  } else {
    return false;
  }
}

CSAPI_TIER2 bool
ModuleParam::set (int it) {
  if (type == MAAATE_TYPE_INT) {
    i = it;
    return true;
  } else {
    return false;
  }
}

CSAPI_TIER2 bool
ModuleParam::set (double rl) {
  if (type == MAAATE_TYPE_REAL) {
    r = rl;
    return true;
  } else {
    return false;
  }
}

CSAPI_TIER2 bool
ModuleParam::set (char * sg) {
  if (type == MAAATE_TYPE_STRING) {
    s = strdup(sg);
    return true;
  } else {
    return false;
  }
}

// overload friend operator <<
// main usage: debugging
CSAPI_TIER2 ostream&
operator<<(ostream& os, ModuleParamRange const & mpr) {
  if (mpr.lowerBound() && mpr.upperBound() && mpr.step()) {
    os << "[" << mpr.lowerBound() << ";" << mpr.upperBound() << "] step " << mpr.step();
  } else if (mpr.lowerBound() && mpr.upperBound()) {
    os << "[" << mpr.lowerBound() << ";" << mpr.upperBound() << "]";
  } else if (mpr.lowerBound()) {
    os << "[" << mpr.lowerBound() << ";infinity]";
  } else if (mpr.upperBound()) {
    os << "[-infinity;" << mpr.lowerBound() << "]";
  }    
  return os;
}

// overload friend operator <<
// main usage: debugging
CSAPI_TIER2 ostream&
operator<<(ostream& os, ModuleParamConstraint & mpc) {
  switch (mpc.Type()) {
  case MAAATE_CONSTRAINT_NONE:
    os << "No constraint" << endl;
    break;
  case MAAATE_CONSTRAINT_VALUE:
    os << mpc.Value() << endl;
    break;
  case MAAATE_CONSTRAINT_RANGE:
    os << mpc.Value()->constraintRange() << endl;
    break;
  default:
    break;
  }
  return os;
}

CSAPI_TIER2 bool
ModuleParamConstraint::within (ModuleParam param) {
  switch (type) {
  case MAAATE_CONSTRAINT_NONE:
    return true;
  case MAAATE_CONSTRAINT_VALUE:
    if (*(constraint->constraintValue()) == (param)) {
      return true;
    }
    break;
  case MAAATE_CONSTRAINT_RANGE:
    if (*(constraint->constraintRange()->lowerBound()) <= (param) &&
	*(constraint->constraintRange()->upperBound()) >= (param)) {
      return true;
    }
    break;
  default:
    break;
  }
  return false;

}

CSAPI_TIER2 ModuleParam *
ModuleParamConstraint::closest (ModuleParam param) {
  if (within(param)) return &param;

  switch (type) {
  case MAAATE_CONSTRAINT_VALUE:
    return constraint->constraintValue();
  case MAAATE_CONSTRAINT_RANGE:
    if (param > * (constraint->constraintRange()->upperBound())) {
      return constraint->constraintRange()->upperBound();
    } else {
      return constraint->constraintRange()->lowerBound();
    }
    break;
  default:
    return NULL;
  }
}

CSAPI_TIER2 double
ModuleParamConstraint::distance (ModuleParam param) {
  if (within(param)) return 0.0;

  switch (type) {
  case MAAATE_CONSTRAINT_VALUE:
    return abs(param - constraint->constraintValue());
  case MAAATE_CONSTRAINT_RANGE:
    if (param > *(constraint->constraintRange()->upperBound())) {
      return (param - constraint->constraintRange()->upperBound());
    } else {
      return (constraint->constraintRange()->lowerBound() - param);
    }
    break;
  default:
    return DBL_MAX;
  }
}

// overload friend operator <<
// main usage: debugging
CSAPI_TIER2 ostream&
operator<<(ostream& os, MaaateConstraint & mc) {
  list<ModuleParamConstraint>::iterator constraintIter;

  if (mc.getConstraints()->empty()) {
    return os;
  }

  for (constraintIter = mc.getConstraints()->begin();
       constraintIter != mc.getConstraints()->end();
       ++constraintIter) {
    os << *constraintIter;
  }

  return os;
}

CSAPI_TIER2 void
MaaateConstraint::addConstraint (ModuleParam * value) {
  constraints.push_back 
    (ModuleParamConstraint(MAAATE_CONSTRAINT_VALUE,
			   new ModuleParamConstraintValue(value)));
}

CSAPI_TIER2 void
MaaateConstraint::addConstraintRange (ModuleParam * lowerBound,
				      ModuleParam * upperBound,
				      ModuleParam * step)
{
  constraints.push_back 
    (ModuleParamConstraint(MAAATE_CONSTRAINT_RANGE,
			   new ModuleParamConstraintValue
			   (new ModuleParamRange
			    (lowerBound, upperBound,
			     ((step == NULL || step->isZero()) ?
			      (ModuleParam *)NULL : step)))));
}

CSAPI_TIER2 void
MaaateConstraint::addConstraintRange (int lowerBound,
				      int upperBound,
				      int step) {
  addConstraintRange(new ModuleParam(lowerBound),
		     new ModuleParam(upperBound),
		     (step == 0 ? (ModuleParam *)NULL :
		      new ModuleParam(step)));
}

CSAPI_TIER2 void
MaaateConstraint::addConstraintRange (double lowerBound,
				      double upperBound,
				      double step) {
  addConstraintRange(new ModuleParam(lowerBound),
		     new ModuleParam(upperBound),
		     (step == 0.0 ? (ModuleParam *)NULL :
		      new ModuleParam(step)));
}

CSAPI_TIER2 void
MaaateConstraint::addConstraintGreaterThan(int i) {
  addConstraintRange(new ModuleParam(i)); 
}

CSAPI_TIER2 void
MaaateConstraint::addConstraintGreaterThan(double d) {
  addConstraintRange(new ModuleParam(d)); 
}

CSAPI_TIER2 void
MaaateConstraint::addConstraintLessThan(int i) {
  addConstraintRange((ModuleParam *)NULL, new ModuleParam(i)); 
}

CSAPI_TIER2 void
MaaateConstraint::addConstraintLessThan(double d) {
  addConstraintRange((ModuleParam *)NULL, new ModuleParam(d)); 
}

CSAPI_TIER2 bool
MaaateConstraint::withinConstraints (ModuleParam param) {
  list<ModuleParamConstraint>::iterator constraintIter;

  if (constraints.empty()) {
    return true;
  }

  for (constraintIter = constraints.begin();
       constraintIter != constraints.end();
       ++constraintIter) {
    if ((*constraintIter).within(param)) {
      return true;
    }
  }

  return false;
}

// return NULL if there is no closer value; eg. if value is already
// the closest, or if there is no distance function
CSAPI_TIER2 ModuleParam *
MaaateConstraint::closestConstraintValue (ModuleParam param) {
  MaaateType type;
  ModuleParam * closest;
  list<ModuleParamConstraint>::iterator constraintIter;
  double thisDistance = 0, minDistance = DBL_MAX;

  // XXX: assume the MaaateType of value is the same as that of the
  // constraint; constraints.getType() would be better
  type = param.getType();
  if (type == MAAATE_TYPE_SOUNDFILE || type == MAAATE_TYPE_SEGMENTTABLE ||
      type == MAAATE_TYPE_SEGMENTDATA) {
    return NULL;
  }

  if (constraints.size() == 0) {
    return NULL;
  }

  closest = new ModuleParam (param);
  for (constraintIter = constraints.begin();
       constraintIter != constraints.end();
       ++constraintIter) {

    if ((*constraintIter).within(param)) {
      return NULL;
    }

    thisDistance = (*constraintIter).distance(param);
    if (thisDistance < minDistance) {
      closest = (*constraintIter).closest(param);
      minDistance = thisDistance;
    }
  }
  return closest;

}

// functions that do nothing for the insane case
CSAPI_TIER2 
list<ModuleParam> * defaultNoop (Module * m) {
  list<ModuleParam> * result = new list<ModuleParam>();
  return result;
};

CSAPI_TIER2 
void suggestNoop (Module * m, list<ModuleParam> * paramsIn) {};
list<ModuleParam> * applyNoop (Module * m, list<ModuleParam> * params_in) {
  list<ModuleParam> * result = new list<ModuleParam>();
  return result;
};

CSAPI_TIER2 
void resetNoop (Module * m) {};

CSAPI_TIER2 
void destroyD (Module * m) {
  m->inputSpecs()->clear();
  m->outputSpecs()->clear();
  //    modParamInSpecs.~list<ModuleParamSpec>();
  //    modParamOutSpecs.~list<ModuleParamSpec>();
};

CSAPI_TIER2 
list<ModuleParam> *
defaultD (Module * m) {
  list<ModuleParamSpec>::iterator iter;
  list<ModuleParam> * plist = new list<ModuleParam>();
  ModuleParam * mp;

  for (iter = m->inputSpecs()->begin();
       iter != m->inputSpecs()->end();
       ++iter) {
    mp = (*iter).defaultValue();
    plist->push_back(*mp);
  }

  return plist;
}

CSAPI_TIER2 
Module::Module(ModuleInitFunc      initF,
	       ModuleDefaultFunc   defaultF,
	       ModuleSuggestValues suggestF,
	       ModuleApplyFunc     applyF,
	       ModuleResetFunc     resetF,
	       ModuleDestroyFunc   destroyF) {
  // execute init function if supplied
  if (initF != NULL) {
    initF(this);
    // XXX: delete whitespace from modName !!!
    
    // set function pointers to other functions
    defaultM = (defaultF==NULL ? defaultD    : defaultF);
    suggestM = (suggestF==NULL ? suggestNoop : suggestF);
    applyM   = (applyF==NULL   ? applyNoop   : applyF);
    destroyM = (destroyF==NULL ? destroyD    : destroyF);
    
    saneM = true;
  } else {
    modName = string("Insane_Module");
    modDesc = string("This module has not been properly initialised.");
    modAuthor = string("");
    modCopyright = string("");
    modUrl = string("");
    modParamInSpecs.clear();
    modParamOutSpecs.clear();
    
    defaultM = defaultNoop;
    suggestM = suggestNoop;
    applyM   = applyNoop;
    destroyM = NULL;
    
    saneM = false;
  }
}

// returns true if params got changed; false otherwise
CSAPI_TIER2 bool
Module::checkConstraints (list<ModuleParam> * paramsIn) {
  list<ModuleParamSpec>::iterator specIter;
  list<ModuleParam>::iterator paramIter;
  MaaateConstraint * constraint;
  ModuleParam * suggest;
  bool change = false;

  for (specIter = modParamInSpecs.begin(), paramIter = paramsIn->begin();
       specIter != modParamInSpecs.end() && paramIter != paramsIn->end();
       ++specIter, ++paramIter) {
    constraint = (*specIter).constraint();

    if (!constraint || constraint->withinConstraints (*paramIter)) {
      continue;
    }

    suggest = constraint->closestConstraintValue (*paramIter);
    if (suggest != NULL) {
      *paramIter = *suggest;
      change = true;
    }
  }
  return change;
}
